/**
 * Copyright (C) 2011 - 101loops.com <dev@101loops.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.crashnote.core.collect.impl;

import com.crashnote.core.collect.BaseCollector;
import com.crashnote.core.collect.impl.excp.ExcpCollector;
import com.crashnote.core.config.Config;
import com.crashnote.core.model.data.*;
import com.crashnote.core.model.log.LogEvt;
import com.crashnote.core.model.types.LogLevel;
import com.crashnote.core.util.IDUtil;

import java.util.Map;

/**
 * Collector to transform one or multiple {@link LogEvt}(s) into a structured data format.
 */
public class LogCollector<C extends Config>
    extends BaseCollector<C> {

    private final ExcpCollector<C> excpCollector;

    // SETUP ======================================================================================

    public LogCollector(final C config) {
        super(config);
        this.excpCollector = createExcpCollector(config);
    }

    // INTERFACE ==================================================================================

    public DataObject collect(final LogEvt evt) {
        return collectEvt(evt);
    }

    public DataArray collect(final LogEvt[] evts) {
        return collectEvts(evts);
    }

    // FACTORY ====================================================================================

    protected ExcpCollector<C> createExcpCollector(final C config) {
        return new ExcpCollector<C>(config);
    }

    // INTERNALS ==================================================================================

    private DataArray collectEvts(final LogEvt[] evts) {
        final DataArray data = createDataArr();
        for (final LogEvt evt : evts) data.add(collectEvt(evt));
        return data;
    }

    private DataObject collectEvt(final LogEvt evt) {
        final DataObject res = createDataObj();
        {
            // meta data
            res.put("m", evt.getMessage());
            res.put("t", evt.getTimeStamp());
            res.put("pg", evt.getLoggerName());
            res.put("th", evt.getThreadName());
            if (evt.getLevel() != LogLevel.ERROR) // assume default is "ERROR"
                res.put("l", evt.getLevel().toString());

            final Object[] msgArgs = evt.getArgs();
            if (msgArgs != null && msgArgs.length > 0) {
                final DataArray args = createDataArr();
                for (final Object obj : msgArgs)
                    args.add(obj.toString());
                res.putArr("args", args);
            }

            if (evt.isExcp()) // give exceptions an own ID (allows referencing)
                res.put("id", IDUtil.createUID());

            // context
            if (evt.isExcp()) {
                final Map ctx = evt.getMDC();
                if (ctx != null && ctx.size() > 0) {
                    final DataObject context = createDataObj();
                    for (final Object key : ctx.keySet())
                        context.put(key.toString(), ctx.get(key).toString());
                    res.putObj("mdc", context);
                }
            }

            // exception data
            final Throwable th = evt.getThrowable();
            if (th != null) res.putArr("excp", collectExcp(th));
        }
        return res;
    }

    private DataArray collectExcp(final Throwable th) {
        return excpCollector.collect(th);
    }
}
