/**
 * Copyright (C) 2011 - 101loops.com <dev@101loops.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.crashnote.core.config;

import com.crashnote.core.log.LogLog;
import com.crashnote.core.util.IDUtil;

import java.util.*;

/**
 * Configuration base class, provides necessary utility methods to manage the actual settings
 * easier and is responsible for managing {@link IConfigChangeListener}(s).
 */
public abstract class ConfigBase {

    /**
     * default prefix for the libraries properties (e.g. in command line)
     */
    public static final String LIB_NAME = "Crashnote";
    public static final String LIB_URL = "https://crashnote.com";

    public static final String LIB_URL_DOCS = LIB_URL + "/docs";
    public static final String LIB_URL_APPS = LIB_URL + "/apps";

    /**
     * time of JVM deployment / start up
     */
    @SuppressWarnings({"FieldMayBeFinal"})
    private long startTime;

    /**
     * unique identifier for environment
     */
    @SuppressWarnings({"FieldMayBeFinal"})
    private long uid;

    /**
     * map to store the configuration settings
     */
    private volatile Map<String, Object> settings = new HashMap<String, Object>();

    /**
     * list of listeners that are notified on any change to the configuration
     */
    private volatile List<IConfigChangeListener> listeners = new ArrayList<IConfigChangeListener>();

    /**
     * default prefix for the libraries properties (e.g. in command line)
     */
    public static final String PROP_PREFIX = LIB_NAME.toLowerCase();

    private final ConfigParser parser;


    // SETUP ======================================================================================

    public ConfigBase() {
        uid = IDUtil.createUID();
        parser = new ConfigParser();
        startTime = new Date().getTime();
    }

    public ConfigBase(final ConfigBase c) {
        this();
        this.uid = c.uid;
        this.startTime = c.startTime;
        this.settings = new HashMap<String, Object>(c.settings);
        this.listeners = new ArrayList<IConfigChangeListener>(c.listeners);
    }

    // INTERFACE ==================================================================================

    /**
     * Called when the config was changed at runtime,
     * the method notifies all listeners about the changes
     */
    public void updateComponentsConfig() {
        getLogger().debug("notifying config listeners about a settings update");
        for (final IConfigChangeListener l : listeners) {
            if (l != null) l.updateConfig(this);
        }
    }

    public void addListener(final IConfigChangeListener listener) {
        if (!listeners.contains(listener)) listeners.add(listener);
    }

    public void removeListener(final IConfigChangeListener listener) {
        listeners.remove(listener);
    }

    public void removeAllListeners() {
        listeners.clear();
    }

    public static String getConfigKey(final String propName) {
        return (propName.startsWith(PROP_PREFIX) ? propName : PROP_PREFIX + "." + propName).toLowerCase();
    }

    // SHARED =====================================================================================

    protected abstract LogLog getLogger();

    // ==== WRITE

    protected void addSetting(final String name, final String value) {
        settings.put(getConfigKey(name), parser.parseString(value));
    }

    protected void addIntSetting(final String name, final String value) {
        try {
            addSetting(name, parser.parseInt(value));
        } catch (IllegalArgumentException e) {
            throw new IllegalArgumentException("parameter [" + name + "] is not a valid number: '" + value + "'", e);
        }
    }

    protected void addSetting(final String name, final int value) {
        settings.put(getConfigKey(name), value);
    }

    protected void addBoolSetting(final String name, final String value) {
        addSetting(name, parser.parseBool(value));
    }

    protected void addSetting(final String name, final boolean value) {
        settings.put(getConfigKey(name), value);
    }

    protected void addSetting(final String name, final Object value) {
        settings.put(getConfigKey(name), value);
    }

    // ==== READ

    protected boolean getBoolSetting(final String name) {
        final Object res = settings.get(getConfigKey(name));
        return (Boolean) ((res == null) ? false : res);
    }

    protected int getIntSetting(final String name) {
        Integer res = -1;
        try {
            res = (Integer) settings.get(getConfigKey(name));
        } catch (ClassCastException e) {
            throw new IllegalStateException("property " + name + " is not a number: " + getStringSetting(name));
        }
        return res;
    }

    protected String getStringSetting(final String name) {
        final Object res = settings.get(getConfigKey(name));
        return (res == null) ? null : res.toString();
    }

    protected Object getSetting(final String name) {
        return settings.get(getConfigKey(name));
    }

    // GET ========================================================================================

    public long getStartTime() {
        return startTime;
    }

    public Long getUID() {
        return uid;
    }

    public List<IConfigChangeListener> getListeners() {
        return listeners;
    }
}
