/**
 * Copyright (C) 2011 - 101loops.com <dev@101loops.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.crashnote.core.report.impl.session;

import com.crashnote.core.config.Config;
import com.crashnote.core.model.log.*;

import java.util.Map;

/**
 * Implementation of a {@link ILogSession} which wraps a {@link SharedLogSession} within a
 * {@link ThreadLocal}. This way each thread that accesses the data has its own copy.
 * <p/>
 * This makes it a perfect candidate for usage in server environments
 * with its request/response threads.
 */
public class LocalLogSession<C extends Config>
        implements ILogSession {

    private final ThreadLocal<SharedLogSession<C>> session;

    // SETUP ======================================================================================

    public LocalLogSession(final C config) {
        session = new InheritableThreadLocal<SharedLogSession<C>>();
        session.set(new SharedLogSession<C>(config));
    }

    // INTERFACE ==================================================================================

    public long getId() {
        return getSession().getId();
    }

    public ILogSession defer() {
        return new SharedLogSession<C>(session.get());
    }

    // ===== EVENTS

    public LogEvt[] getEvents() {
        return getSession().getEvents();
    }

    public void addEvt(final LogEvt<?> evt) {
        getSession().addEvt(evt);
    }

    public boolean isEmpty() {
        return getSession().isEmpty();
    }

    // ===== CONTEXT

    public Map<String, Object> getContext() {
        return getSession().getContext();
    }

    public void put(final String key, final Object val) {
        getSession().put(key, val);
    }

    public void remove(final String key) {
        getSession().remove(key);
    }

    public void clear() {
        getSession().clear();
    }

    // INTERNALS ==================================================================================

    public ILogSession getSession() {
        return session.get();
    }
}
