/**
 * Copyright (C) 2011 - 101loops.com <dev@101loops.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.crashnote.servlet.collect;

import com.crashnote.core.collect.BaseCollector;
import com.crashnote.core.model.data.DataObject;
import com.crashnote.core.util.*;
import com.crashnote.servlet.config.ServletConfig;

import javax.servlet.http.*;
import java.security.Principal;
import java.util.*;

/**
 * Collector to transform a HTTP request into a structured data format.
 */
public class RequestCollector
    extends BaseCollector<ServletConfig> {

    protected String[] requestFilters;
    protected boolean skipHeaderData;
    protected boolean skipSessionData;

    // SETUP ======================================================================================

    public RequestCollector(final ServletConfig config) {
        super(config);
        this.requestFilters = config.getRequestFilters();
        this.skipHeaderData = !config.getCollectHeaderData();
        this.skipSessionData = !config.getCollectSessionData();
    }

    // INTERFACE ==================================================================================

    public DataObject collect(final HttpServletRequest req) {
        return collect(req, createDataObj());
    }

    public DataObject collect(final HttpServletRequest req, final DataObject data) {
        collectReqBase(req, data);
        {
            data.put("id", IDUtil.createUID());
            data.put("start", new Date().getTime());

            data.putObj("params", collectReqParams(req));
            if (!skipHeaderData) data.putObj("header", collectReqHeader(req));
            data.putObj("session", collectReqSession(req));

        }
        return data;
    }

    // SHARED =====================================================================================

    protected DataObject collectReqBase(final HttpServletRequest req, final DataObject data) {
        {
            data.put("mth", req.getMethod());
            data.put("url", req.getRequestURL().toString());

            final String userIP = req.getRemoteAddr();
            data.put("iph", ChksumUtil.hash(userIP));

            final Principal principal = req.getUserPrincipal();
            if (principal != null) data.put("principal", principal.getName());
        }
        return data;
    }

    protected DataObject collectReqSession(final HttpServletRequest req) {
        final DataObject data = createDataObj();
        {
            final HttpSession session = req.getSession();

            // collect basic info
            data.put("id", session.getId());
            data.put("t", session.getCreationTime());

            // collect session data
            if (!skipSessionData) {
                final DataObject content = createDataObj();
                {
                    final Enumeration names = session.getAttributeNames();
                    while (names.hasMoreElements()) {
                        final String name = names.nextElement().toString();
                        final String value = session.getAttribute(name).toString();
                        data.put(name, value);
                    }
                }
                data.put("cnt", content);
            }
        }
        return data;
    }

    protected DataObject collectReqHeader(final HttpServletRequest req) {
        final DataObject data = createDataObj();
        {
            final Enumeration names = req.getHeaderNames();
            while (names.hasMoreElements()) {
                final String name = names.nextElement().toString();

                final List<String> values = new ArrayList<String>();
                final Enumeration header = req.getHeaders(name);
                while (header.hasMoreElements()) {
                    final String value = header.nextElement().toString();
                    values.add(value);
                }
                if (values.size() == 1) data.put(name, values.get(0));
                else data.put(name, createDataArr(values));
            }
        }
        return data;
    }

    protected DataObject collectReqParams(final HttpServletRequest req) {
        final DataObject data = createDataObj();
        {
            final Enumeration names = req.getParameterNames();
            while (names.hasMoreElements()) {
                final String name = names.nextElement().toString();
                if (!isFiltered(name)) {
                    final String value = req.getParameter(name);
                    data.put(name, value);
                }
            }
        }
        return data;
    }

    // INTERNALS ==================================================================================

    private boolean isFiltered(final String name) {
        for (final String filter : requestFilters)
            if (name.matches(filter)) return true;
        return false;
    }
}
